@@ -9,11 +9,16 @@
 //! A number of environment variables are available to globally configure how
 //! this crate will invoke `pkg-config`:
 //!
-//! * `PKG_CONFIG_ALLOW_CROSS` - if this variable is not set, then `pkg-config`
-//!   will automatically be disabled for all cross compiles.
 //! * `FOO_NO_PKG_CONFIG` - if set, this will disable running `pkg-config` when
 //!   probing for the library named `foo`.
 //!
+//! * `PKG_CONFIG_ALLOW_CROSS` - The `pkg-config` command usually doesn't
+//!   support cross-compilation, and this crate prevents it from selecting
+//!   incompatible versions of libraries.
+//!   Setting `PKG_CONFIG_ALLOW_CROSS=1` disables this protection, which is
+//!   likely to cause linking errors, unless `pkg-config` has been configured
+//!   to use appropriate sysroot and search paths for the target platform.
+//!
 //! There are also a number of environment variables which can configure how a
 //! library is linked to (dynamically vs statically). These variables control
 //! whether the `--static` flag is passed. Note that this behavior can be
@@ -63,38 +68,30 @@
 
 #![doc(html_root_url = "https://docs.rs/pkg-config/0.3")]
 
-#[allow(unused_imports)] // Required for Rust <1.23
-use std::ascii::AsciiExt;
 use std::collections::HashMap;
 use std::env;
 use std::error;
 use std::ffi::{OsStr, OsString};
 use std::fmt;
 use std::io;
-use std::path::{PathBuf, Path};
+use std::ops::{Bound, RangeBounds};
+use std::path::PathBuf;
 use std::process::{Command, Output};
 use std::str;
 
-pub fn target_supported() -> bool {
-    let target = env::var("TARGET").unwrap_or_else(|_| String::new());
-    let host = env::var("HOST").unwrap_or_else(|_| String::new());
-
-    // Only use pkg-config in host == target situations by default (allowing an
-    // override).
-    (host == target || env::var_os("PKG_CONFIG_ALLOW_CROSS").is_some())
-}
-
-#[derive(Clone, Default)]
+#[derive(Clone, Debug)]
 pub struct Config {
     statik: Option<bool>,
-    atleast_version: Option<String>,
+    min_version: Bound<String>,
+    max_version: Bound<String>,
     extra_args: Vec<OsString>,
     cargo_metadata: bool,
     env_metadata: bool,
     print_system_libs: bool,
+    print_system_cflags: bool,
 }
 
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub struct Library {
     pub libs: Vec<String>,
     pub link_paths: Vec<PathBuf>,
@@ -113,9 +110,11 @@ pub enum Error {
     /// Contains the name of the responsible environment variable.
     EnvNoPkgConfig(String),
 
-    /// Cross compilation detected.
+    /// Detected cross compilation without a custom sysroot.
     ///
-    /// Override with `PKG_CONFIG_ALLOW_CROSS=1`.
+    /// Ignore the error with `PKG_CONFIG_ALLOW_CROSS=1`,
+    /// which may let `pkg-config` select libraries
+    /// for the host's architecture instead of the target's.
     CrossCompilation,
 
     /// Failed to run `pkg-config`.
@@ -133,98 +132,71 @@ pub enum Error {
     __Nonexhaustive,
 }
 
-impl error::Error for Error {
-    fn description(&self) -> &str {
-        match *self {
-            Error::EnvNoPkgConfig(_) => "pkg-config requested to be aborted",
-            Error::CrossCompilation => {
-                "pkg-config doesn't handle cross compilation. \
-                 Use PKG_CONFIG_ALLOW_CROSS=1 to override"
-            }
-            Error::Command { .. } => "failed to run pkg-config",
-            Error::Failure { .. } => "pkg-config did not exit sucessfully",
-            Error::__Nonexhaustive => panic!(),
-        }
-    }
+impl error::Error for Error {}
 
-    fn cause(&self) -> Option<&error::Error> {
-        match *self {
-            Error::Command { ref cause, .. } => Some(cause),
-            _ => None,
-        }
-    }
-}
-
-// Workaround for temporary lack of impl Debug for Output in stable std
-struct OutputDebugger<'a>(&'a Output);
-
-// Lifted from 1.7 std
-impl<'a> fmt::Debug for OutputDebugger<'a> {
-    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-        let stdout_utf8 = str::from_utf8(&self.0.stdout);
-        let stdout_debug: &fmt::Debug = match stdout_utf8 {
-            Ok(ref str) => str,
-            Err(_) => &self.0.stdout
-        };
-
-        let stderr_utf8 = str::from_utf8(&self.0.stderr);
-        let stderr_debug: &fmt::Debug = match stderr_utf8 {
-            Ok(ref str) => str,
-            Err(_) => &self.0.stderr
-        };
-
-        fmt.debug_struct("Output")
-           .field("status", &self.0.status)
-           .field("stdout", stdout_debug)
-           .field("stderr", stderr_debug)
-           .finish()
-    }
-}
-
-// Workaround for temporary lack of impl Debug for Output in stable std, continued
 impl fmt::Debug for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
-        match *self {
-            Error::EnvNoPkgConfig(ref name) => {
-                f.debug_tuple("EnvNoPkgConfig")
-                 .field(name)
-                 .finish()
-            }
-            Error::CrossCompilation => write!(f, "CrossCompilation"),
-            Error::Command { ref command, ref cause } => {
-                f.debug_struct("Command")
-                 .field("command", command)
-                 .field("cause", cause)
-                 .finish()
-            }
-            Error::Failure { ref command, ref output } => {
-                f.debug_struct("Failure")
-                 .field("command", command)
-                 .field("output", &OutputDebugger(output))
-                 .finish()
-            }
-            Error::__Nonexhaustive => panic!(),
-        }
+        // Failed `unwrap()` prints Debug representation, but the default debug format lacks helpful instructions for the end users
+        <Error as fmt::Display>::fmt(self, f)
     }
 }
 
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
         match *self {
-            Error::EnvNoPkgConfig(ref name) => {
-                write!(f, "Aborted because {} is set", name)
+            Error::EnvNoPkgConfig(ref name) => write!(f, "Aborted because {} is set", name),
+            Error::CrossCompilation => f.write_str(
+                "pkg-config has not been configured to support cross-compilation.\n\
+                \n\
+                Install a sysroot for the target platform and configure it via\n\
+                PKG_CONFIG_SYSROOT_DIR and PKG_CONFIG_PATH, or install a\n\
+                cross-compiling wrapper for pkg-config and set it via\n\
+                PKG_CONFIG environment variable.",
+            ),
+            Error::Command {
+                ref command,
+                ref cause,
+            } => {
+                match cause.kind() {
+                    io::ErrorKind::NotFound => {
+                        let crate_name = std::env::var("CARGO_PKG_NAME");
+                        let crate_name = crate_name.as_deref().unwrap_or("sys");
+                        let instructions = if cfg!(target_os = "macos") || cfg!(target_os = "ios") {
+                            "Try `brew install pkg-config` if you have Homebrew.\n"
+                        } else if cfg!(unix) {
+                            "Try `apt install pkg-config`, or `yum install pkg-config`,\n\
+                            or `pkg install pkg-config` depending on your distribution.\n"
+                        } else {
+                            "" // There's no easy fix for Windows users
+                        };
+                        write!(f, "Could not run `{command}`\n\
+                        The pkg-config command could not be found.\n\
+                        \n\
+                        Most likely, you need to install a pkg-config package for your OS.\n\
+                        {instructions}\
+                        \n\
+                        If you've already installed it, ensure the pkg-config command is one of the\n\
+                        directories in the PATH environment variable.\n\
+                        \n\
+                        If you did not expect this build to link to a pre-installed system library,\n\
+                        then check documentation of the {crate_name} crate for an option to\n\
+                        build the library from source, or disable features or dependencies\n\
+                        that require pkg-config.", command = command, instructions = instructions, crate_name = crate_name)
+                    }
+                    _ => write!(f, "Failed to run command `{}`, because: {}", command, cause),
+                }
             }
-            Error::CrossCompilation => {
-                write!(f, "Cross compilation detected. \
-                       Use PKG_CONFIG_ALLOW_CROSS=1 to override")
-            }
-            Error::Command { ref command, ref cause } => {
-                write!(f, "Failed to run `{}`: {}", command, cause)
-            }
-            Error::Failure { ref command, ref output } => {
+            Error::Failure {
+                ref command,
+                ref output,
+            } => {
                 let stdout = str::from_utf8(&output.stdout).unwrap();
                 let stderr = str::from_utf8(&output.stderr).unwrap();
-                write!(f, "`{}` did not exit successfully: {}", command, output.status)?;
+                write!(
+                    f,
+                    "`{}` did not exit successfully: {}",
+                    command, output.status
+                )?;
                 if !stdout.is_empty() {
                     write!(f, "\n--- stdout\n{}", stdout)?;
                 }
@@ -250,12 +222,17 @@ pub fn probe_library(name: &str) -> Result<Library, Error> {
 }
 
 /// Run `pkg-config` to get the value of a variable from a package using
-/// --variable.
+/// `--variable`.
+///
+/// The content of `PKG_CONFIG_SYSROOT_DIR` is not injected in paths that are
+/// returned by `pkg-config --variable`, which makes them unsuitable to use
+/// during cross-compilation unless specifically designed to be used
+/// at that time.
 pub fn get_variable(package: &str, variable: &str) -> Result<String, Error> {
     let arg = format!("--variable={}", variable);
     let cfg = Config::new();
     let out = run(cfg.command(package, &[&arg]))?;
-    Ok(str::from_utf8(&out).unwrap().trim_right().to_owned())
+    Ok(str::from_utf8(&out).unwrap().trim_end().to_owned())
 }
 
 impl Config {
@@ -264,11 +241,13 @@ impl Config {
     pub fn new() -> Config {
         Config {
             statik: None,
-            atleast_version: None,
+            min_version: Bound::Unbounded,
+            max_version: Bound::Unbounded,
             extra_args: vec![],
+            print_system_cflags: true,
             print_system_libs: true,
             cargo_metadata: true,
-            env_metadata: false,
+            env_metadata: true,
         }
     }
 
@@ -283,7 +262,33 @@ impl Config {
 
     /// Indicate that the library must be at least version `vers`.
     pub fn atleast_version(&mut self, vers: &str) -> &mut Config {
-        self.atleast_version = Some(vers.to_string());
+        self.min_version = Bound::Included(vers.to_string());
+        self.max_version = Bound::Unbounded;
+        self
+    }
+
+    /// Indicate that the library must be equal to version `vers`.
+    pub fn exactly_version(&mut self, vers: &str) -> &mut Config {
+        self.min_version = Bound::Included(vers.to_string());
+        self.max_version = Bound::Included(vers.to_string());
+        self
+    }
+
+    /// Indicate that the library's version must be in `range`.
+    pub fn range_version<'a, R>(&mut self, range: R) -> &mut Config
+    where
+        R: RangeBounds<&'a str>,
+    {
+        self.min_version = match range.start_bound() {
+            Bound::Included(vers) => Bound::Included(vers.to_string()),
+            Bound::Excluded(vers) => Bound::Excluded(vers.to_string()),
+            Bound::Unbounded => Bound::Unbounded,
+        };
+        self.max_version = match range.end_bound() {
+            Bound::Included(vers) => Bound::Included(vers.to_string()),
+            Bound::Excluded(vers) => Bound::Excluded(vers.to_string()),
+            Bound::Unbounded => Bound::Unbounded,
+        };
         self
     }
 
@@ -304,7 +309,7 @@ impl Config {
 
     /// Define whether metadata should be emitted for cargo allowing to
     /// automatically rebuild when environment variables change. Defaults to
-    /// `false`.
+    /// `true`.
     pub fn env_metadata(&mut self, env_metadata: bool) -> &mut Config {
         self.env_metadata = env_metadata;
         self
@@ -319,6 +324,15 @@ impl Config {
         self
     }
 
+    /// Enable or disable the `PKG_CONFIG_ALLOW_SYSTEM_CFLAGS` environment
+    /// variable.
+    ///
+    /// This env var is enabled by default.
+    pub fn print_system_cflags(&mut self, print: bool) -> &mut Config {
+        self.print_system_cflags = print;
+        self
+    }
+
     /// Deprecated in favor fo the `probe` function
     #[doc(hidden)]
     pub fn find(&self, name: &str) -> Result<Library, String> {
@@ -332,8 +346,8 @@ impl Config {
     pub fn probe(&self, name: &str) -> Result<Library, Error> {
         let abort_var_name = format!("{}_NO_PKG_CONFIG", envify(name));
         if self.env_var_os(&abort_var_name).is_some() {
-            return Err(Error::EnvNoPkgConfig(abort_var_name))
-        } else if !target_supported() {
+            return Err(Error::EnvNoPkgConfig(abort_var_name));
+        } else if !self.target_supported() {
             return Err(Error::CrossCompilation);
         }
 
@@ -348,34 +362,60 @@ impl Config {
         Ok(library)
     }
 
+    pub fn target_supported(&self) -> bool {
+        let target = env::var_os("TARGET").unwrap_or_default();
+        let host = env::var_os("HOST").unwrap_or_default();
+
+        // Only use pkg-config in host == target situations by default (allowing an
+        // override).
+        if host == target {
+            return true;
+        }
+
+        // pkg-config may not be aware of cross-compilation, and require
+        // a wrapper script that sets up platform-specific prefixes.
+        match self.targetted_env_var("PKG_CONFIG_ALLOW_CROSS") {
+            // don't use pkg-config if explicitly disabled
+            Some(ref val) if val == "0" => false,
+            Some(_) => true,
+            None => {
+                // if not disabled, and pkg-config is customized,
+                // then assume it's prepared for cross-compilation
+                self.targetted_env_var("PKG_CONFIG").is_some()
+                    || self.targetted_env_var("PKG_CONFIG_SYSROOT_DIR").is_some()
+            }
+        }
+    }
+
     /// Deprecated in favor of the top level `get_variable` function
     #[doc(hidden)]
     pub fn get_variable(package: &str, variable: &str) -> Result<String, String> {
         get_variable(package, variable).map_err(|e| e.to_string())
     }
 
-    fn targetted_env_var(&self, var_base: &str) -> Result<String, env::VarError> {
-        if let Ok(target) = env::var("TARGET") {
-            let host = env::var("HOST")?;
-            let kind = if host == target { "HOST" } else { "TARGET" };
-            let target_u = target.replace("-", "_");
+    fn targetted_env_var(&self, var_base: &str) -> Option<OsString> {
+        match (env::var("TARGET"), env::var("HOST")) {
+            (Ok(target), Ok(host)) => {
+                let kind = if host == target { "HOST" } else { "TARGET" };
+                let target_u = target.replace("-", "_");
 
-            self.env_var(&format!("{}_{}", var_base, target))
-                .or_else(|_| self.env_var(&format!("{}_{}", var_base, target_u)))
-                .or_else(|_| self.env_var(&format!("{}_{}", kind, var_base)))
-                .or_else(|_| self.env_var(var_base))
-        } else {
-            self.env_var(var_base)
+                self.env_var_os(&format!("{}_{}", var_base, target))
+                    .or_else(|| self.env_var_os(&format!("{}_{}", var_base, target_u)))
+                    .or_else(|| self.env_var_os(&format!("{}_{}", kind, var_base)))
+                    .or_else(|| self.env_var_os(var_base))
+            }
+            (Err(env::VarError::NotPresent), _) | (_, Err(env::VarError::NotPresent)) => {
+                self.env_var_os(var_base)
+            }
+            (Err(env::VarError::NotUnicode(s)), _) | (_, Err(env::VarError::NotUnicode(s))) => {
+                panic!(
+                    "HOST or TARGET environment variable is not valid unicode: {:?}",
+                    s
+                )
+            }
         }
     }
 
-    fn env_var(&self, name: &str) -> Result<String, env::VarError> {
-        if self.env_metadata {
-            println!("cargo:rerun-if-env-changed={}", name);
-        }
-        env::var(name)
-    }
-
     fn env_var_os(&self, name: &str) -> Option<OsString> {
         if self.env_metadata {
             println!("cargo:rerun-if-env-changed={}", name);
@@ -388,35 +428,48 @@ impl Config {
     }
 
     fn command(&self, name: &str, args: &[&str]) -> Command {
-        let exe = self.env_var("PKG_CONFIG").unwrap_or_else(|_| String::from("pkg-config"));
+        let exe = self
+            .targetted_env_var("PKG_CONFIG")
+            .unwrap_or_else(|| OsString::from("pkg-config"));
         let mut cmd = Command::new(exe);
         if self.is_static(name) {
             cmd.arg("--static");
         }
-        if let Ok(target) = env::var("TARGET") {
-            if target.contains("msvc") {
-                cmd.arg("--msvc-syntax");
-            }
-        }
-        cmd.args(args)
-           .args(&self.extra_args);
+        cmd.args(args).args(&self.extra_args);
 
-        if let Ok(value) = self.targetted_env_var("PKG_CONFIG_PATH") {
+        if let Some(value) = self.targetted_env_var("PKG_CONFIG_PATH") {
             cmd.env("PKG_CONFIG_PATH", value);
         }
-        if let Ok(value) = self.targetted_env_var("PKG_CONFIG_LIBDIR") {
+        if let Some(value) = self.targetted_env_var("PKG_CONFIG_LIBDIR") {
             cmd.env("PKG_CONFIG_LIBDIR", value);
         }
-        if let Ok(value) = self.targetted_env_var("PKG_CONFIG_SYSROOT_DIR") {
+        if let Some(value) = self.targetted_env_var("PKG_CONFIG_SYSROOT_DIR") {
             cmd.env("PKG_CONFIG_SYSROOT_DIR", value);
         }
         if self.print_system_libs {
             cmd.env("PKG_CONFIG_ALLOW_SYSTEM_LIBS", "1");
         }
-        if let Some(ref version) = self.atleast_version {
-            cmd.arg(&format!("{} >= {}", name, version));
-        } else {
-            cmd.arg(name);
+        if self.print_system_cflags {
+            cmd.env("PKG_CONFIG_ALLOW_SYSTEM_CFLAGS", "1");
+        }
+        cmd.arg(name);
+        match self.min_version {
+            Bound::Included(ref version) => {
+                cmd.arg(&format!("{} >= {}", name, version));
+            }
+            Bound::Excluded(ref version) => {
+                cmd.arg(&format!("{} > {}", name, version));
+            }
+            _ => (),
+        }
+        match self.max_version {
+            Bound::Included(ref version) => {
+                cmd.arg(&format!("{} <= {}", name, version));
+            }
+            Bound::Excluded(ref version) => {
+                cmd.arg(&format!("{} < {}", name, version));
+            }
+            _ => (),
         }
         cmd
     }
@@ -443,6 +496,22 @@ impl Config {
     }
 }
 
+// Implement Default manualy since Bound does not implement Default.
+impl Default for Config {
+    fn default() -> Config {
+        Config {
+            statik: None,
+            min_version: Bound::Unbounded,
+            max_version: Bound::Unbounded,
+            extra_args: vec![],
+            print_system_cflags: false,
+            print_system_libs: false,
+            cargo_metadata: false,
+            env_metadata: false,
+        }
+    }
+}
+
 impl Library {
     fn new() -> Library {
         Library {
@@ -458,15 +527,43 @@ impl Library {
     }
 
     fn parse_libs_cflags(&mut self, name: &str, output: &[u8], config: &Config) {
-        let words = split_flags(output);
-        let parts = words.iter()
-                          .filter(|l| l.len() > 2)
-                          .map(|arg| (&arg[0..2], &arg[2..]))
-                          .collect::<Vec<_>>();
+        let mut is_msvc = false;
+        if let Ok(target) = env::var("TARGET") {
+            if target.contains("msvc") {
+                is_msvc = true;
+            }
+        }
+
+        let system_roots = if cfg!(target_os = "macos") {
+            vec![PathBuf::from("/Library"), PathBuf::from("/System")]
+        } else {
+            let sysroot = config
+                .env_var_os("PKG_CONFIG_SYSROOT_DIR")
+                .or_else(|| config.env_var_os("SYSROOT"))
+                .map(PathBuf::from);
+
+            if cfg!(target_os = "windows") {
+                if let Some(sysroot) = sysroot {
+                    vec![sysroot]
+                } else {
+                    vec![]
+                }
+            } else {
+                vec![sysroot.unwrap_or_else(|| PathBuf::from("/usr"))]
+            }
+        };
 
         let mut dirs = Vec::new();
         let statik = config.is_static(name);
-        for &(flag, val) in &parts {
+
+        let words = split_flags(output);
+
+        // Handle single-character arguments like `-I/usr/include`
+        let parts = words
+            .iter()
+            .filter(|l| l.len() > 2)
+            .map(|arg| (&arg[0..2], &arg[2..]));
+        for (flag, val) in parts {
             match flag {
                 "-L" => {
                     let meta = format!("rustc-link-search=native={}", val);
@@ -483,64 +580,77 @@ impl Library {
                     self.include_paths.push(PathBuf::from(val));
                 }
                 "-l" => {
-                    self.libs.push(val.to_string());
-                    if statik && is_static_available(val, &dirs) {
+                    // These are provided by the CRT with MSVC
+                    if is_msvc && ["m", "c", "pthread"].contains(&val) {
+                        continue;
+                    }
+
+                    if statik && is_static_available(val, &system_roots, &dirs) {
                         let meta = format!("rustc-link-lib=static={}", val);
                         config.print_metadata(&meta);
                     } else {
                         let meta = format!("rustc-link-lib={}", val);
                         config.print_metadata(&meta);
                     }
+
+                    self.libs.push(val.to_string());
                 }
                 "-D" => {
-                    let mut iter = val.split("=");
-                    self.defines.insert(iter.next().unwrap().to_owned(), iter.next().map(|s| s.to_owned()));
+                    let mut iter = val.split('=');
+                    self.defines.insert(
+                        iter.next().unwrap().to_owned(),
+                        iter.next().map(|s| s.to_owned()),
+                    );
                 }
                 _ => {}
             }
         }
 
-        let mut iter = words.iter()
-                            .flat_map(|arg| if arg.starts_with("-Wl,") {
-                                 arg[4..].split(',').collect()
-                             } else {
-                                 vec![arg.as_ref()]
-                             });
-        while let Some(part) = iter.next() {
-            if part != "-framework" {
-                continue
+        // Handle multi-character arguments with space-separated value like `-framework foo`
+        let mut iter = words.iter().flat_map(|arg| {
+            if arg.starts_with("-Wl,") {
+                arg[4..].split(',').collect()
+            } else {
+                vec![arg.as_ref()]
             }
-            if let Some(lib) = iter.next() {
-                let meta = format!("rustc-link-lib=framework={}", lib);
-                config.print_metadata(&meta);
-                self.frameworks.push(lib.to_string());
+        });
+        while let Some(part) = iter.next() {
+            match part {
+                "-framework" => {
+                    if let Some(lib) = iter.next() {
+                        let meta = format!("rustc-link-lib=framework={}", lib);
+                        config.print_metadata(&meta);
+                        self.frameworks.push(lib.to_string());
+                    }
+                }
+                "-isystem" | "-iquote" | "-idirafter" => {
+                    if let Some(inc) = iter.next() {
+                        self.include_paths.push(PathBuf::from(inc));
+                    }
+                }
+                _ => (),
             }
         }
     }
 
     fn parse_modversion(&mut self, output: &str) {
-        self.version.push_str(output.trim());
+        self.version.push_str(output.lines().nth(0).unwrap().trim());
     }
 }
 
 fn envify(name: &str) -> String {
-    name.chars().map(|c| c.to_ascii_uppercase()).map(|c| {
-        if c == '-' {'_'} else {c}
-    }).collect()
+    name.chars()
+        .map(|c| c.to_ascii_uppercase())
+        .map(|c| if c == '-' { '_' } else { c })
+        .collect()
 }
 
 /// System libraries should only be linked dynamically
-fn is_static_available(name: &str, dirs: &[PathBuf]) -> bool {
+fn is_static_available(name: &str, system_roots: &[PathBuf], dirs: &[PathBuf]) -> bool {
     let libname = format!("lib{}.a", name);
-    let system_roots = if cfg!(target_os = "macos") {
-        vec![Path::new("/Library"), Path::new("/System")]
-    } else {
-        vec![Path::new("/usr")]
-    };
 
     dirs.iter().any(|dir| {
-        !system_roots.iter().any(|sys| dir.starts_with(sys)) &&
-        dir.join(&libname).exists()
+        !system_roots.iter().any(|sys| dir.starts_with(sys)) && dir.join(&libname).exists()
     })
 }
 
@@ -552,13 +662,13 @@ fn run(mut cmd: Command) -> Result<Vec<u8>, Error> {
             } else {
                 Err(Error::Failure {
                     command: format!("{:?}", cmd),
-                    output: output,
+                    output,
                 })
             }
         }
         Err(cause) => Err(Error::Command {
             command: format!("{:?}", cmd),
-            cause: cause,
+            cause,
         }),
     }
 }
@@ -581,9 +691,7 @@ fn split_flags(output: &[u8]) -> Vec<String> {
                 escaped = false;
                 word.push(b);
             }
-            b'\\' => {
-                escaped = true
-            }
+            b'\\' => escaped = true,
             b'\t' | b'\n' | b'\r' | b' ' => {
                 if !word.is_empty() {
                     words.push(String::from_utf8(word).unwrap());
@@ -604,19 +712,57 @@ fn split_flags(output: &[u8]) -> Vec<String> {
 #[test]
 #[cfg(target_os = "macos")]
 fn system_library_mac_test() {
-    assert!(!is_static_available("PluginManager", &[PathBuf::from("/Library/Frameworks")]));
-    assert!(!is_static_available("python2.7", &[PathBuf::from("/System/Library/Frameworks/Python.framework/Versions/2.7/lib/python2.7/config")]));
-    assert!(!is_static_available("ffi_convenience", &[PathBuf::from("/Library/Ruby/Gems/2.0.0/gems/ffi-1.9.10/ext/ffi_c/libffi-x86_64/.libs")]));
+    use std::path::Path;
+
+    let system_roots = vec![PathBuf::from("/Library"), PathBuf::from("/System")];
+
+    assert!(!is_static_available(
+        "PluginManager",
+        &system_roots,
+        &[PathBuf::from("/Library/Frameworks")]
+    ));
+    assert!(!is_static_available(
+        "python2.7",
+        &system_roots,
+        &[PathBuf::from(
+            "/System/Library/Frameworks/Python.framework/Versions/2.7/lib/python2.7/config"
+        )]
+    ));
+    assert!(!is_static_available(
+        "ffi_convenience",
+        &system_roots,
+        &[PathBuf::from(
+            "/Library/Ruby/Gems/2.0.0/gems/ffi-1.9.10/ext/ffi_c/libffi-x86_64/.libs"
+        )]
+    ));
 
     // Homebrew is in /usr/local, and it's not a part of the OS
     if Path::new("/usr/local/lib/libpng16.a").exists() {
-        assert!(is_static_available("png16", &[PathBuf::from("/usr/local/lib")]));
+        assert!(is_static_available(
+            "png16",
+            &system_roots,
+            &[PathBuf::from("/usr/local/lib")]
+        ));
+
+        let libpng = Config::new()
+            .range_version("1".."99")
+            .probe("libpng16")
+            .unwrap();
+        assert!(libpng.version.find('\n').is_none());
     }
 }
 
 #[test]
 #[cfg(target_os = "linux")]
 fn system_library_linux_test() {
-    assert!(!is_static_available("util", &[PathBuf::from("/usr/lib/x86_64-linux-gnu")]));
-    assert!(!is_static_available("dialog", &[PathBuf::from("/usr/lib")]));
+    assert!(!is_static_available(
+        "util",
+        &[PathBuf::from("/usr")],
+        &[PathBuf::from("/usr/lib/x86_64-linux-gnu")]
+    ));
+    assert!(!is_static_available(
+        "dialog",
+        &[PathBuf::from("/usr")],
+        &[PathBuf::from("/usr/lib")]
+    ));
 }
